import time
from functions import *
import functions

class Character():
    # x and y are pixel coordinates
    x = None
    y = None
    # If this character is bigger than a tile, bump it up by offsety and left by offsetx
    offsetx = 0
    offsety = 0
    in_transit = False
    
    speed = 3

    def __init__(self, pos = False):
        if pos:
            self.set_pos(pos)

    def get_drawpoint(self):
        try:
            return (self.x - TILESIZE/2 - self.offsetx,self.y - TILESIZE/2 - self.offsety)
        except:
            return False

    def get_xy(self):
        return (self.x,self.y)

    def set_pos(self,coords):
        self.x = coords[1] * TILESIZE + TILESIZE/2
        self.y = coords[0] * TILESIZE + TILESIZE/2
        self.xf, self.yf = self.x, self.y
        self.starttile = coords
        self.facing = 'r'

    def draw(self):
        try:
            if self.facing == 'l':
                WINDOW.blit(pygame.transform.flip(self.img, True, False), self.get_drawpoint())
            else:
                WINDOW.blit(self.img, self.get_drawpoint())
        except:
            print "Didn't work. Are all elements of the level placed?"
            pygame.quit()
            sys.exit()

    def move(self, d, speed = False):
        if not speed:
            speed = self.speed
        if d == 'u':
            self.yf -= speed
            self.y = int(self.yf)
        elif d == 'd':
            self.yf += speed
            self.y = int(self.yf)
        elif d == 'l':
            self.xf -= speed
            self.x = int(self.xf)
        elif d == 'r':
            self.xf += speed
            self.x = int(self.xf)
        if speed > SLOWSPEED and d in ('l','r'):
            self.facing = d 

    # TODO: finish me
    def move_tile(self, d):
        self.move(d)
        if (self.x, self.y) == functions.tile_center(self.get_tile()):
            self.in_transit = False
        
    def get_tile(self):
        try:
            return (self.y//TILESIZE, self.x//TILESIZE)
        except:
            return False

    def get_side(self, l):
        if l == 'u':
            return self.y - OCCUPYSIZE/2
        elif l == 'd':
            return self.y + OCCUPYSIZE/2
        elif l == 'l':
            return self.x - OCCUPYSIZE/2
        elif l == 'r':
            return self.x + OCCUPYSIZE/2

    def animate(self, gameobj, n=1, otheranim = False):
        """For in-place, paused-engine animations"""
        if otheranim:
            thisanim = otheranim
        else:
            try:
                thisanim = self.anim
            except:
                return
        for i in range(n):
            for frame in thisanim:
                self.img = frame
                gameobj.draw_all()
                pygame.display.update()
                time.sleep(ANIMATIONINTERVAL)
        self.img = thisanim[0]

        

class PlayerCharacter(Character):
    speed = PCSPEED
    img = pygame.image.load(imgpath('pc.png'))
    newframe = PCFRAMESPERFRAME
    framecounter = 0
    walkframe = 0
    offsety = PCIMGHEIGHT - TILESIZE

    walkanim = [pygame.image.load(imgpath('pc_walk_1.png')),
                pygame.image.load(imgpath('pc.png')),
                pygame.image.load(imgpath('pc_walk_2.png')),
                pygame.image.load(imgpath('pc.png'))]

    def draw(self):
        self.framestep()
#        super(PlayerCharacter, self).draw()
        Character.draw(self)

    def framestep(self):
        try:
            if self.framecounter >= self.newframe:
                self.framecounter = 0
                self.walkframe += 1
                if self.walkframe == 4:
                    self.walkframe = 0
                self.img = self.walkanim[self.walkframe]
        except:
            print "Debug: couldn't step"
            return

    def move(self, d, reduced = False, wraparound = False):
        if wraparound:
            Character.move(self, d, speed = SLOWSPEED)
        elif reduced:
            # Set speed at regular over root 2
            Character.move(self, d, speed = self.speed / 1.4)
            self.framecounter += 0.5
        else:
            Character.move(self, d)
            self.framecounter += 1


class Robot(Character):
    starttile = False
    speed = ROBOTSPEED
    happyanim = [pygame.image.load(imgpath('robot.png')),
                pygame.image.load(imgpath('robot_2.png'))]
    sadanim = [pygame.image.load(imgpath('robotsad.png')),
                pygame.image.load(imgpath('robotsad_2.png'))]
    curblink = 0
    blinkinterval = 20
    counter = 0
    img = happyanim[0]
    doiblink = True

    def blink(self, happy):
        if self.counter >= self.blinkinterval:
            self.counter = 0
            self.curblink += 1
            if self.curblink >= 2:
                self.curblink = 0
        if happy:
            self.img = self.happyanim[self.curblink]
        else:
            self.img = self.sadanim[self.curblink]
        self.counter += 1

    def behave(self, gameobj):
        # Find all the hazards
        board = gameobj.board.board
        hazards = gameobj.hazards[:]
        if gameobj.bomb.live:
            hazards.append(gameobj.bomb)

        if self.doiblink:
            if len(hazards):
                self.blink(False)
            else:
                self.blink(True)

        if not self.starttile:
            return
        if self.in_transit:
            self.move_tile(self.transit_direction)
            return
        here = self.get_tile()
        # Disarm the bomb if sitting on it
        if here == gameobj.bomb.get_tile() and gameobj.bomb.live:
            gameobj.bomb.disarm()
            self.animate(gameobj)
        # ... or fix a pipe!
        for e in gameobj.effects:
            if here == e.get_tile() and gameobj.board[here] in BURSTPIPES:
                SOUND['repair'].play()
                self.animate(gameobj)
                gameobj.fix_pipe(here)
        for h in gameobj.hazards:
            if here == h.get_tile():
                SOUND['repair'].play()
                gameobj.hazards.remove(h)

        # Find the closest hazard and move to it
        if len(hazards):
            paths = []
            for h in hazards:
                paths.append(self.find_route(h.get_tile(), gameobj))
            if len(paths):
                paths.sort()
                for i in xrange(len(paths)):
                    if len(paths[i]):
                        self.transit_direction = paths[i][0]
                        if not self.in_transit:
                            self.play_sound()
                            self.in_transit = True
                        break

        # If all hazards are taken care of, move back home
        if not len(hazards) and self.get_tile() != self.starttile:
            try:
                self.transit_direction = self.find_route(self.starttile, gameobj)[0]
                self.in_transit = True
            except:
                pass
            
    def find_route(self, dest, gameobj):
        board = gameobj.board.board
        if not dest: return []
        eligible = gameobj.board.get_eligible_tiles(self.get_tile())
#        eligible = set((r, c) for r in xrange(len(board)) for c in xrange(len(board[r])) if board[r][c] not in BOTBLOCKED and not gameobj.tile_is_occupied((r,c), bybot = False))
        tilestoremove = []
        for tile in eligible:
            if gameobj.tile_is_occupied(tile, bybot = False):
                tilestoremove.append(tile)
        for tile in tilestoremove:
            eligible.remove(tile)
        eligible.add(dest)
        return functions.bfs_short_path(self.get_tile(), dest, eligible)
        
    def play_sound(self):
        SOUND['robotmove'].play()

class RobotArm(Robot):
    img = pygame.image.load(imgpath('robotarm.png'))
    anim = [pygame.image.load(imgpath('robotarm.png')),
            pygame.image.load(imgpath('robotarm_2.png')),
            pygame.image.load(imgpath('robotarm_3.png'))]
    doiblink = False

    def play_sound(self):
        pass
