import time
import pygame
from functions import *
import functions as f

class Thing():

    def __init__(self, (row, col)):
        self.row, self.col = row, col

    def get_tile(self):
        try:
            return (self.row, self.col)
        except:
            return False

    def get_drawpoint(self):
        try:
            return (self.col * TILESIZE, self.row * TILESIZE)
        except:
            return False

    def draw(self):
        try:
            WINDOW.blit(self.img, self.get_drawpoint())
        except:
            pass

    def is_active(self):
        try:
            return self.active
        except:
            return True

    def animate(self, gameobj, n=1, otheranim = False):
        """For in-place, paused-engine animations"""
        if otheranim:
            thisanim = otheranim
        else:
            try:
                thisanim = self.anim
            except:
                return
        for i in range(n):
            for frame in thisanim:
                self.img = frame
                gameobj.draw_all()
                pygame.display.update()
                time.sleep(ANIMATIONINTERVAL)
        self.img = thisanim[0]

class SpecialTile(Thing):

    def draw(self):
        try:
            self.anim
            self.counter
            self.step
            self.frame
        except:
            return
        if self.counter >= self.step:
            self.counter = 0
            self.frame += 1
            if self.frame >= len(self.anim):
                self.frame = 0
        self.img = self.anim[self.frame]
        self.counter += 1
        WINDOW.blit(self.img, self.get_drawpoint())

class Goal(SpecialTile):
    anim = GOALANIM
    counter = 0
    step = 9
    frame = 0

class Scorched(SpecialTile):
    anim = FIREANIM
    counter = 0
    step = 5
    frame = 0

class Lever(Thing):
    value = None
    anim = [pygame.image.load(imgpath('lever.png')),
            pygame.image.load(imgpath('lever_2.png')),
            pygame.image.load(imgpath('lever_3.png'))]
    img = anim[0]
    dooranim = [pygame.image.load(imgpath('opendoor.png')),
            pygame.image.load(imgpath('door_3.png')),
            pygame.image.load(imgpath('door_2.png')),
            pygame.image.load(imgpath('closeddoor.png'))]

    def __init__(self, loc, value):
#        if dooropen: self.dooropen = True
        self.value = value
        self.row, self.col = loc
        self.doors = []
#        self.door = doorloc

    def add_door(self, doorloc, isopen = False):
        self.doors.append([doorloc,isopen])

    def pull(self, gameobj):
        SOUND['lever'].play()
        self.animate(gameobj)
        self.act(gameobj)

    def act(self, gameobj):
        for door in self.doors:
            if door[1] == False:
                door[1] = True
                newdoor = LEVEROPENDOOR[self.value]
                self.dooranimate(gameobj, door[0], 'open')
            else:
                door[1] = False
                newdoor = LEVERDOOR[self.value]
                self.dooranimate(gameobj, door[0], 'close')
            gameobj.board.change_tile(door[0], newdoor)

    def dooranimate(self, gameobj, tile, action):
        if action == 'open':
            animtouse = reversed(self.dooranim)
        elif action == 'close':
            animtouse = self.dooranim
        SOUND['door'].play()
        for frame in animtouse:
            gameobj.draw_all()
            gameobj.place_tile(frame, tile)
            pygame.display.update()
            time.sleep(ANIMATIONINTERVAL/2)
        

class DestructibleSwitch(Lever):
    img = SWITCHIMG
    intact = True

    def get_xy(self):
        try:
            return ((self.col * TILESIZE + TILESIZE/2),(self.row * TILESIZE + TILESIZE/2))
        except:
            return False

    def pull(self):
        pass

    def destroy(self, gameobj):
        if self.intact:
            self.intact = False
            self.img = BROKENSWITCHIMG
            SOUND['switch'].play()
            time.sleep(0.4)
            self.act(gameobj)

class Bomb(Thing):
    timer = 0
    live = False
    img = pygame.image.load(imgpath('bomb.png'))

    def __init__(self):
        self.live = False

    def arm(self, (row, col)):
        self.row, self.col = row, col
        self.timer = 0
        self.live = True
        SOUND['setbomb'].play()

    def disarm(self):
        if self.live:
            SOUND['disarmbomb'].play()
            time.sleep(0.5)
            self.live = False

    def draw(self):
        if self.live:
            WINDOW.blit(self.img, self.get_drawpoint())

    def explode(self, gameobj):
        if self.live:
            self.live = False
            self.exploding = True
        elif self.exploding:
            return
        if not self.get_tile():
            return
        
        blowntiles = gameobj.board.get_eligible_tiles(self.get_tile(), agent = 'bomb')
        for tile in blowntiles:
            # Check to see if BFS finds it. If not, don't use it!
            if len(functions.bfs_short_path(self.get_tile(), tile, blowntiles)) or tile == self.get_tile():
                gameobj.hazards.append(Scorched(tile))
        SOUND['explode'].play()

        for a in reversed(range(0,256,4)):
            WHITEOUTSURF.set_alpha(a)
            WHITEOUTSURF.fill((255,255,255,a))
            WINDOW.blit(WHITEOUTSURF,(0,0))
            pygame.display.update()
            time.sleep(0.01)
            WINDOW.blit(BACKGROUND, (0,0))
            gameobj.draw_all()
        self.exploding = False

    def tick(self, gameobj):
        if not self.live:
            return
        self.timer += 1
        if self.timer >= BOMBTIMER:
            self.explode(gameobj)

    def is_available(self, level):
        return False if level < BOMBAVAILABLE else True


class Effect():
    """
    Handles radial effects like toxic sludge.
    """

    speed = 1

    def __init__(self, xy):
        self.x, self.y = xy
        self.r = 0
        self.active = True

    def explosion_contains(self, xy):
        x, y = xy
        if ((x - self.x)**2 + (y - self.y)**2) ** 0.5 < self.r:
            return True
        return False

    def is_active(self):
        return self.active

    def get_tile(self):
        return f.xy2tile((self.x,self.y))

    def draw(self):
        pass

    def deactivate(self):
        self.active = False

    def update(self):
        if self.active:
            self.grow()

    def grow(self):
        if self.r < SLUDGEMAXRADIUS:
            self.r += self.speed
            self.draw()

class Sludge(Effect):
    speed = SLUDGESPEED

    def draw(self):
        pygame.draw.circle(EFFECTSURF, cSLUDGE, (self.x,self.y), int(self.r))
#        WINDOW.blit(EFFECTSURF, (0,0))

